@@ -4,7 +4,7 @@ use warnings;
 require 5.008;
 
 package Net::SIP;
-our $VERSION = '0.54';
+our $VERSION = '0.59';
 
 # this includes nearly everything else
 use Net::SIP::Simple ();
@@ -30,8 +30,10 @@ BEGIN {
 		Net::SIP::Simple::RTP
 		Net::SIP::Dispatcher
 		Net::SIP::Dispatcher::Eventloop
+		Net::SIP::Redirect
 		Net::SIP::Registrar
 		Net::SIP::StatelessProxy
+		Net::SIP::Blocker
 		Net::SIP::ReceiveChain
 		Net::SIP::Authorize
 		Net::SIP::Endpoint
@@ -71,7 +73,10 @@ sub import {
 			Net::SIP::Debug->export_to_level(1,$class,':DEFAULT')
 		} elsif ( $tag eq ':alias' ) {
 			$class->export_to_level(1,$class,$tag);
-		} elsif ( $tag =~m{^debug=(.*)}i ) {
+		} elsif ( $tag =~m{rtp[=:](\d+)-(\d+)}i ) {
+			$Net::SIP::Util::RTP_MIN_PORT = $1;
+			$Net::SIP::Util::RTP_MAX_PORT = $2;
+		} elsif ( $tag =~m{^debug[=:](.*)}i ) {
 			Net::SIP::Debug->level($1);
 		} elsif ( first { $_ eq $tag } @EXPORT_OK ) {
 			# from the predefined list
